import mkdirp from 'mkdirp';
import fs from 'fs';
import _  from 'lodash';
import path from 'path';
import jsTools from './file/js-tools.js';
import jsonTools from './file/json-tools.js';

const BIG_NUM = 999999999999;
/**
 * get current directory
 */
const getCurrentDir = ()=> (
   process.cwd()
);
/**
 * create directory if directory not exist
 * @param {string}dir
 */
const createDirNotExist = (dir)=> {
  if (!fs.existsSync(dir)) {
    mkdirp.sync(dir);
  }
};
/**
 * Returns trailer of file name.
 * @param {string} name file fullName that contain path
 * @return {string|null}
 */
let getTrailerAfterDot = (name)=> {
  if (typeof name !== 'string') return null;
  let parts = name.split('.');
  if (parts.length < 2) return null;
  return parts[parts.length - 1].toLowerCase();
};

const openFile = (path , type , mode)=>{
  type = type || getTrailerAfterDot(path);
  let result;
  if(type === 'js')
    result = jsTools.open(path,mode);
  else if(type === 'json')
    result = jsonTools.open(path,mode);
  else
    result = openFileAsPlainString(path);
  return result;
};

/**
 * open file and return string
 * @param {string} file path
 * @return {string} file content
 */
const openFileAsPlainString = (path)=>(
  fs.readFileSync(path, 'utf8')
)
/**
 * return string that remove Bom from input string
 * @param {string}str
 * @returns {string}
 */
const stripBom = (str)=> (
   str.charCodeAt(0) === 0xFEFF ? str.slice(1) : str
);

/**
 * calculate path depth
 * @param {string}fullPath path
 * @returns {Number}
 */
const directoryDepth = (fullPath)=> {
  assert(typeof fullPath === 'string');
  return _.compact(fullPath.split(path.sep)).length;
};

/**
 * get real path
 * @param {string}path
 * @return {string|null}
 */
const realPath = (path)=> {
  let realPath = null;
  try {
    realPath = process.browser ? path : fs.realpathSync(path);
  } catch (e) {
    return null;
  }

  return realPath;
};

/**
 * if path is symbol link ,return real path
 * @param {string}path
 * @returns {string|null}
 */
const unsymbolLink = (path)=> {
  if (!path) return null;
  let stats = null;
  try {
    stats = fs.lstatSync(path);
  } catch (e) {
    return null;
  }

  if (!stats) return null;
  if (stats.isSymbolicLink()) {
    let realPath = fileTools.realPath(path);

    return unsymbolLink(realPath);
  } else {
    return stats.isDirectory() ? path : null;
  }
};

/**
 * get max directory depth
 * @returns {Number}
 */
const maxDirectoryDepth = ()=> {
  let depth = parseInt(process.env.MOCOOLKA_GLOBALIZE_MAX_DEPTH, 10);
  if (isNaN(depth)) depth = BIG_NUM;
  depth = Math.max(1, depth);
  return depth;
};

/**
 * if the depth that current directory in root directory more than max directory depth return true
 * @param {string}currentDir
 * @param {string} rootDir
 * @returns {boolean}
 */
const IsExceedMaxDirectoryDepth = (currentDir, rootDir)=> {
  let depthRoot = directoryDepth(rootDir);
  let depthCurrent = directoryDepth(currentDir);
  if ((depthCurrent - depthRoot) > maxDirectoryDepth())
    return true;
  else
    return false;

};

let fileTools = {
  getCurrentDir,
  createDirNotExist,
  getTrailerAfterDot,
  directoryDepth,
  realPath,
  unsymbolLink,
  maxDirectoryDepth,
  IsExceedMaxDirectoryDepth,
  stripBom,
  openFile,
  openFileAsPlainString,
};
export default fileTools;

