import moduleTools from './module-tools.js';
import path from 'path';
import fs from 'fs';
import fileTools from './file-tools.js';
import assert from 'assert';

const ENGLISH = 'en';
const PSEUDO_TAG = 'zz';
let MY_ROOT = process.cwd();
let INTL_DIR = path.join(MY_ROOT, 'intl');

/**
 * @param {string} lang Supported languages in CLDR notation
 * Returns directory path for the language.
 */
let intlDir = (lang)=> {
  lang = lang || ENGLISH;
  return path.join(INTL_DIR, lang);
};

let KEY_HEADERS = ['msg'];

const hashKeys = (path)=> {
  let trailer = null;
  return !(headerIncluded(path, KEY_HEADERS)
  || (trailer = fileTools.getTrailerAfterDot(path)) === 'txt'
  || trailer === 'json'
  || trailer === 'yml'
  || trailer === 'yaml'
  || path.indexOf(PSEUDO_TAG) === 0);
};

/**
 * verify name's header be allowed
 * @param {string} name to be checked
 * @param {Array} headersAllowed a list of strings to check
 * @returns {boolean}
 */
let headerIncluded = (name, headersAllowed)=> {
  let matched = false;
  if (Array.isArray(headersAllowed)) {
    headersAllowed.map(header => {
      if (matched) return;
      matched = (name.indexOf(header) === 0);
    });
  } else if (typeof headersAllowed === 'string') {
    matched = (name.indexOf(headersAllowed) === 0);
  }

  return matched;
};
/**
 * @param {string}rootPath
 */
const getRootDir = (rootPath)=> {
  rootPath = rootPath || path.join(fileTools.getCurrentDir(), '..');
  let validPath = true;
  let rootStats;
  try {
    rootStats = fs.statSync(path.normalize(rootPath));
  } catch (e) {
    validPath = false;
  }

  assert(validPath, '*** setRootDir: Root path invalid: ' + rootPath);
  if (!rootStats.isDirectory()) validPath = false;
  assert(validPath,	'*** setRootDir: Root path is not a directory: ' + rootPath.toString());
  let files = null;
  try {
    files = fs.readdirSync(rootPath);
  } catch (e) {
    validPath = false;
  }

  validPath = (validPath && !!files);
  if (validPath) {
    let intlDirFound = false;
    files.forEach(function (item) {
      if (intlDirFound) return;
      if (item === 'intl') intlDirFound = true;
    });

    validPath = intlDirFound;
  }

  assert(validPath,	'*** setRootDir: Intl dir not found under: ' + rootPath.toString());
  return {
    root: rootPath,
    intl: path.join(rootPath, 'intl'),
    cldr: path.join(rootPath, 'cldr'),
    log: path.join(rootPath, 'logs'),
  };
};

/**
 * enumerate message sync
 * @param {string}rootDir
 * @param {boolean}checkNodeModules
 * @param {function}callback
 */
const enumerateMsgSync = (rootDir,  checkNodeModules, callback)=> {
  moduleTools.enumerateFilesSync(rootDir, [], ['json'], checkNodeModules, callback);
};

/**
 * Supported languages in CLDR notation
 */
const I18nTools = {
  enumerateMsgSync,
  getRootDir,
  ENGLISH,
  hashKeys,
};

export default I18nTools;
